"use strict";
// src/rules/sort-keys.ts
Object.defineProperty(exports, "__esModule", { value: true });
exports.sortKeysRule = void 0;
exports.sortKeysRule = {
  meta: {
    type: "suggestion",
    docs: {
      description: "Enforce sorted keys in objects",
      recommended: false,
    },
    fixable: "code",
    messages: {
      sortKeys:
        "Expected object keys to be in {{orderText}} order. '{{thisName}}' should be before '{{prevName}}' .",
    },
    schema: [
      {
        enum: ["asc", "desc"],
      },
      {
        type: "object",
        properties: {
          caseSensitive: { type: "boolean" },
          natural: { type: "boolean" },
          minKeys: { type: "number" },
          allowLineSeparatedGroups: { type: "boolean" },
        },
        additionalProperties: false,
      },
    ],
  },
  create(context) {
    const options = context.options[0] || "asc";
    const config = context.options[1] || {};
    const order = options === "asc" ? "ascending" : "descending";
    const caseSensitive = config.caseSensitive ?? true;
    const natural = config.natural ?? false;
    const minKeys = config.minKeys ?? 0;
    const allowLineSeparatedGroups = config.allowLineSeparatedGroups ?? false;
    return {
      ObjectExpression(node) {
        const properties = node.properties.filter(
          (prop) =>
            prop.type === "Property" &&
            !prop.computed &&
            (prop.key.type === "Identifier" ||
              (prop.key.type === "Literal" &&
                typeof prop.key.value === "string")),
        );
        if (properties.length < minKeys) {
          return;
        }
        let previousKey = null;
        properties.forEach((prop, index) => {
          let currentKey;
          if (prop.key.type === "Identifier") {
            currentKey = caseSensitive
              ? prop.key.name
              : prop.key.name.toLowerCase();
          } else if (
            prop.key.type === "Literal" &&
            typeof prop.key.value === "string"
          ) {
            currentKey = caseSensitive
              ? prop.key.value
              : prop.key.value.toLowerCase();
          } else {
            // Skip if key type is not Identifier or string Literal
            return;
          }
          // Detect new group if allowed
          if (allowLineSeparatedGroups && index > 0) {
            const sourceCode = context.getSourceCode();
            const prevProperty = properties[index - 1];
            const prevEndLine = sourceCode.getLineAndCharacterFor(
              prevProperty.range[1],
            ).line;
            const currentStartLine = sourceCode.getLineAndCharacterFor(
              prop.range[0],
            ).line;
            if (currentStartLine > prevEndLine + 1) {
              previousKey = null;
            }
          }
          if (previousKey !== null) {
            let isSorted = false;
            if (natural) {
              isSorted =
                options === "asc"
                  ? previousKey.localeCompare(currentKey, undefined, {
                      numeric: true,
                    }) <= 0
                  : previousKey.localeCompare(currentKey, undefined, {
                      numeric: true,
                    }) >= 0;
            } else {
              isSorted =
                options === "asc"
                  ? previousKey <= currentKey
                  : previousKey >= currentKey;
            }
            if (!isSorted) {
              context.report({
                node: prop.key,
                messageId: "sortKeys",
                data: {
                  order,
                  thisName:
                    prop.key.type === "Identifier"
                      ? prop.key.name
                      : prop.key.value,
                  prevName: previousKey,
                },
                fix: (fixer) => {
                  const sortedKeys = properties
                    .map((p) =>
                      p.key.type === "Identifier" ? p.key.name : p.key.value,
                    )
                    .sort((a, b) =>
                      options === "asc"
                        ? a.localeCompare(b, undefined, {
                            numeric: true,
                            sensitivity: caseSensitive ? "case" : "base",
                          })
                        : b.localeCompare(a, undefined, {
                            numeric: true,
                            sensitivity: caseSensitive ? "case" : "base",
                          }),
                    );
                  const sortedProperties = sortedKeys
                    .map((key) =>
                      properties.find((p) =>
                        p.key.type === "Identifier"
                          ? p.key.name === key
                          : p.key.value === key,
                      ),
                    )
                    .filter((p) => p !== undefined);
                  const sortedCode = sortedProperties
                    .map((p) => context.getSourceCode().getText(p))
                    .join(",\n");
                  return fixer.replaceTextRange(
                    [node.range[0] + 1, node.range[1] - 1],
                    `\n${sortedCode}\n`,
                  );
                },
              });
            }
          }
          previousKey = currentKey;
        });
      },
    };
  },
};
