"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.sortKeysRule = void 0;
exports.sortKeysRule = {
    meta: {
        type: "suggestion",
        docs: {
            description: "Enforce sorted keys in objects",
        },
        fixable: "code",
        messages: {
            sortKeys: 'Expected object keys to be in {{ order }} order. "{{ thisName }}" should be before "{{ prevName }}".',
        },
        schema: [
            {
                type: "string",
                enum: ["asc", "desc"],
            },
            {
                type: "object",
                properties: {
                    caseSensitive: { type: "boolean" },
                    natural: { type: "boolean" },
                    minKeys: { type: "number" },
                    allowLineSeparatedGroups: { type: "boolean" },
                },
                additionalProperties: false,
            },
        ],
    },
    defaultOptions: [
        "asc",
        {
            caseSensitive: true,
            natural: false,
            minKeys: 0,
            allowLineSeparatedGroups: false,
        },
    ],
    create(context) {
        const [order, configOptions = {}] = context.options;
        const caseSensitive = configOptions.caseSensitive ?? true;
        const isAscending = order === "asc";
        const displayOrder = isAscending ? "ascending" : "descending";
        const natural = configOptions.natural ?? false;
        const minKeys = configOptions.minKeys ?? 0;
        const allowLineSeparatedGroups = configOptions.allowLineSeparatedGroups ?? false;
        return {
            ObjectExpression(node) {
                const properties = node.properties.filter((prop) => prop.type === "Property");
                if (properties.length < minKeys)
                    return;
                const isAscending = order === "asc";
                const compareKeys = (a, b) => {
                    if (!caseSensitive) {
                        a = a.toLowerCase();
                        b = b.toLowerCase();
                    }
                    if (natural) {
                        return isAscending
                            ? a.localeCompare(b, undefined, { numeric: true })
                            : b.localeCompare(a, undefined, { numeric: true });
                    }
                    return isAscending ? (a < b ? -1 : 1) : a > b ? -1 : 1;
                };
                let lastLine = properties[0].loc.end.line;
                properties.forEach((prop, index) => {
                    if (index === 0)
                        return;
                    const prevProp = properties[index - 1];
                    if (allowLineSeparatedGroups &&
                        prevProp.loc.end.line < prop.loc.start.line - 1) {
                        lastLine = prop.loc.end.line;
                        return;
                    }
                    const thisName = prop.key.type === "Identifier"
                        ? prop.key.name
                        : prop.key.type === "Literal" &&
                            typeof prop.key.value === "string"
                            ? prop.key.value
                            : "";
                    const prevName = prevProp.key.type === "Identifier"
                        ? prevProp.key.name
                        : prevProp.key.type === "Literal" &&
                            typeof prevProp.key.value === "string"
                            ? prevProp.key.value
                            : "";
                    if (compareKeys(prevName, thisName) > 0) {
                        context.report({
                            node: prop.key,
                            messageId: "sortKeys",
                            data: { order: displayOrder, thisName, prevName },
                            fix: (fixer) => {
                                const sortedProps = [...properties].sort((a, b) => {
                                    const aName = a.key.type === "Identifier"
                                        ? a.key.name
                                        : a.key.type === "Literal" &&
                                            typeof a.key.value === "string"
                                            ? a.key.value
                                            : "";
                                    const bName = b.key.type === "Identifier"
                                        ? b.key.name
                                        : b.key.type === "Literal" &&
                                            typeof b.key.value === "string"
                                            ? b.key.value
                                            : "";
                                    return compareKeys(aName, bName);
                                });
                                const sourceCode = context.getSourceCode();
                                const fixedCode = sortedProps
                                    .map((p) => sourceCode.getText(p))
                                    .join(",\n  "); // Adjust indentation and commas
                                return fixer.replaceTextRange([
                                    node.properties[0].range[0],
                                    node.properties[node.properties.length - 1].range[1],
                                ], fixedCode);
                            },
                        });
                    }
                });
            },
        };
    },
};
